@@ -50,6 +50,13 @@ FullPinyinEditor::insert (gint ch)
     if (G_UNLIKELY (m_text.length () >= MAX_PINYIN_LEN))
         return TRUE;
 
+#if 0
+    /* for i/u/v mode */
+    if (m_text.empty () &&
+	(ch == 'i' || ch == 'u' || ch == 'v'))
+      return FALSE;
+#endif
+
     m_text.insert (m_cursor++, ch);
 
     updatePinyin ();
@@ -63,6 +70,18 @@ FullPinyinEditor::processKeyEvent (guint keyval,
                                             guint keycode,
                                             guint modifiers)
 {
+    /* handle 'A' - 'Z' key */
+    if (G_UNLIKELY (IBUS_A <= keyval && keyval <= IBUS_Z)) {
+        if (cmshm_filter (modifiers) == 0) {
+
+            if (m_text.empty ())
+                return FALSE;
+
+            if (insert (keyval))
+                return TRUE;
+        }
+    }
+
     return PinyinEditor::processKeyEvent (keyval, keycode, modifiers);
 }
 
@@ -86,7 +105,8 @@ void
 FullPinyinEditor::updateAuxiliaryText (void)
 {
     if (G_UNLIKELY (m_text.empty ())) {
-        if (DISPLAY_STYLE_TRADITIONAL == m_config.displayStyle ())
+        if (DISPLAY_STYLE_TRADITIONAL == m_config.displayStyle () ||
+            DISPLAY_STYLE_COMPATIBILITY == m_config.displayStyle ())
             hideAuxiliaryText ();
         if (DISPLAY_STYLE_COMPACT == m_config.displayStyle ())
             hidePreeditText ();
@@ -105,7 +125,8 @@ FullPinyinEditor::updateAuxiliaryText (void)
     m_buffer << p;
 
     StaticText text (m_buffer);
-    if (DISPLAY_STYLE_TRADITIONAL == m_config.displayStyle ())
+    if (DISPLAY_STYLE_TRADITIONAL == m_config.displayStyle () ||
+        DISPLAY_STYLE_COMPATIBILITY == m_config.displayStyle ())
         Editor::updateAuxiliaryText (text, TRUE);
     if (DISPLAY_STYLE_COMPACT == m_config.displayStyle ())
         Editor::updatePreeditText (text, 0, TRUE);
